#!/usr/bin/env python

#############################################################################
##
## Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
## All rights reserved.
## Contact: Nokia Corporation (qt-info@nokia.com)
##
## This file is part of the documentation of Qt. It was originally
## published as part of Qt Quarterly.
##
## $QT_BEGIN_LICENSE:LGPL$
## Commercial Usage
## Licensees holding valid Qt Commercial licenses may use this file in
## accordance with the Qt Commercial License Agreement provided with the
## Software or, alternatively, in accordance with the terms contained in
## a written agreement between you and Nokia.
##
## GNU Lesser General Public License Usage
## Alternatively, this file may be used under the terms of the GNU Lesser
## General Public License version 2.1 as published by the Free Software
## Foundation and appearing in the file LICENSE.LGPL included in the
## packaging of this file.  Please review the following information to
## ensure the GNU Lesser General Public License version 2.1 requirements
## will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
##
## In addition, as a special exception, Nokia gives you certain additional
## rights.  These rights are described in the Nokia Qt LGPL Exception
## version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
##
## GNU General Public License Usage
## Alternatively, this file may be used under the terms of the GNU
## General Public License version 3.0 as published by the Free Software
## Foundation and appearing in the file LICENSE.GPL included in the
## packaging of this file.  Please review the following information to
## ensure the GNU General Public License version 3.0 requirements will be
## met: http://www.gnu.org/copyleft/gpl.html.
##
## If you have questions regarding the use of this file, please contact
## Nokia at qt-info@nokia.com.
## $QT_END_LICENSE$
##
#############################################################################

import sys

from PyQt4.QtCore import *
from PyQt4.QtGui import *
from PyQt4.QtMobility.QtLocation import *

import landmarks_rc

class MapWindow(QMainWindow):

    def __init__(self, service):
    
        QMainWindow.__init__(self)
        
        self.service = service
        self.path = ""
        
        self.scene = QGraphicsScene()
        view = QGraphicsView()
        view.setScene(self.scene)
        
        self.mapManager = service.mappingManager()
        
        if not self.mapManager:
            QMessageBox.critical(self,
                self.tr("Landmarks"),
                self.tr('No mapping facility is available.'))
            QApplication.quit(1)
        
        self.landmarkManager = QLandmarkManager()
        
        self.geoMap = QGraphicsGeoMap(self.mapManager)
        self.geoMap.resize(300, 300)
        self.geoMap.setMapType(self.geoMap.SatelliteMapDay)
        self.geoMap.setCenter(QGeoCoordinate(0, 0))
        self.geoMap.setZoomLevel(1)
        self.scene.addItem(self.geoMap)
        
        # Landmark controls
        
        fileMenu = self.menuBar().addMenu(self.tr("&File"))
        importAction = fileMenu.addAction(self.tr("&Import Landmarks..."), self.importLandmarks)
        exportAction = fileMenu.addAction(self.tr("&Export Landmarks..."), self.exportLandmarks)
        quitAction = fileMenu.addAction(self.tr("E&xit"), self.close, QKeySequence(QKeySequence.Quit))
        
        manageMenu = self.menuBar().addMenu(self.tr("&Manage"))
        clearAction = manageMenu.addAction(self.tr("&Clear Landmarks"), self.clearLandmarks)
        
        self.landmarkCombo = QComboBox()
        self.landmarkCombo.setSizePolicy(
            QSizePolicy(QSizePolicy.MinimumExpanding, QSizePolicy.Preferred))
        self.landmarkCombo.currentIndexChanged.connect(self.showLandmark)
        self.updateLandmarks()
        
        widget = QWidget()
        layout = QFormLayout(widget)
        layout.addRow(view)
        layout.addRow(self.tr("&Landmark:"), self.landmarkCombo)
        
        if self.landmarkManager.landmarks():
            self.showLandmark(0)
        
        self.setCentralWidget(widget)
        self.setWindowTitle(self.tr("Landmarks"))
    
    @pyqtSlot(int)
    def showLandmark(self, index):
    
        if index == -1:
            return
        
        landmark = self.landmarkCombo.itemData(index).toPyObject()
        self.geoMap.setCenter(landmark.coordinate())
        self.geoMap.setZoomLevel(14)
    
    def updateLandmarks(self):
    
        self.landmarkCombo.clear()
        
        for obj in self.geoMap.mapObjects():
            self.geoMap.removeMapObject(obj)
        
        for landmark in self.landmarkManager.landmarks():
        
            self.landmarkCombo.addItem(landmark.name(), QVariant(landmark))
            pixmapObj = QGeoMapPixmapObject()
            pixmapObj.setCoordinate(landmark.coordinate())
            pixmapObj.setOffset(QPoint(-2, -30))
            pixmapObj.setPixmap(QPixmap(":Resources/landmark.png"))
            self.geoMap.addMapObject(pixmapObj)
    
    def importLandmarks(self):
    
        path = QFileDialog.getOpenFileName(self, self.tr("Import Landmarks"),
            self.path, self.tr("Landmark files (*.gpx *.lmx)"))
        
        if path.isEmpty():
            return
        
        f = QFile(path)
        if not self.landmarkManager.importLandmarks(f):
            QMessageBox.warning(self, self.tr("Landmarks"),
                self.tr('Failed to import landmarks: "%1"').arg(self.landmarkManager.errorString()))
        else:
            self.updateLandmarks()
    
    def exportLandmarks(self):
    
        path = QFileDialog.getSaveFileName(self, self.tr("Export Landmarks"),
            self.path, self.tr("Landmark files (*.gpx *.lmx)"))
        
        if path.isEmpty():
            return
        
        suffix = os.path.splitext(path)[1]
        suffix = suffix[:1].upper() + suffix[1:]
        
        f = QFile(path)
        if not self.landmarkManager.exportLandmarks(f, suffix):
            QMessageBox.warning(self, self.tr("Landmarks"),
                self.tr('Failed to export landmarks: "%1"').arg(self.landmarkManager.errorString()))
    
    def clearLandmarks(self):
    
        answer = QMessageBox.question(self, self.tr("Landmarks"),
            self.tr("Remove all stored landmarks, including those stored "
                    "persistently in the database?"),
            QMessageBox.Yes, QMessageBox.No)
        
        if answer == QMessageBox.Yes:
            self.landmarkManager.removeLandmarks(self.landmarkManager.landmarks())
            self.updateLandmarks()


if __name__ == "__main__":

    app = QApplication(sys.argv)
    services = QGeoServiceProvider.availableServiceProviders()
    if not services:
        QMessageBox.critical(None,
            app.translate("landmarks", "Landmarks"),
            app.translate("landmarks",
                          "Failed to find any map services. Please ensure that "
                          "the location services plugins for Qt Mobility have "
                          "been built and, if necessary, set the QT_PLUGIN_PATH "
                          "environment variable to the location of the Qt Mobility "
                          "plugins directory."))
        sys.exit(1)
    
    service = QGeoServiceProvider("nokia")
    if service.error() != service.NoError:
        QMessageBox.critical(None,
            app.translate("landmarks", "Landmarks"),
            app.translate("landmarks",
                          "The service failed with the following error:\n"
                          "%1").arg(services.errorString()))
        sys.exit(1)
    
    window = MapWindow(service)
    window.show()
    sys.exit(app.exec_())
