/****************************************************************************
**
** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (qt-info@nokia.com)
**
** This file is part of the documentation of Qt. It was originally
** published as part of Qt Quarterly.
**
** $QT_BEGIN_LICENSE:LGPL$
** Commercial Usage
** Licensees holding valid Qt Commercial licenses may use this file in
** accordance with the Qt Commercial License Agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and Nokia.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Nokia gives you certain additional
** rights.  These rights are described in the Nokia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
**
** If you have questions regarding the use of this file, please contact
** Nokia at qt-info@nokia.com.
** $QT_END_LICENSE$
**
****************************************************************************/

#include <QtGui>
#include <QGeoMapPixmapObject>
#include <QLandmark>
#include "mapwindow.h"

using namespace QtMobility;

MapWindow::MapWindow(QGeoServiceProvider *service, QWidget *parent)
    : QMainWindow(parent), service(service), path("")
{
    scene = new QGraphicsScene(this);
    view = new QGraphicsView();
    view->setScene(scene);

    mapManager = service->mappingManager();

    if (!mapManager) {
        QMessageBox::critical(this, tr("Landmarks"),
                              tr("No mapping facility is available."));
        QApplication::quit();
    }

    landmarkManager = new QLandmarkManager(this);

    geoMap = new QGraphicsGeoMap(mapManager);
    geoMap->resize(300, 300);
    geoMap->setMapType(QGraphicsGeoMap::SatelliteMapDay);
    geoMap->setCenter(QGeoCoordinate(0, 0));
    geoMap->setZoomLevel(1);
    scene->addItem(geoMap);

    // Landmark controls

    QMenu *fileMenu = menuBar()->addMenu(tr("&File"));
    fileMenu->addAction(tr("&Import Landmarks..."), this, SLOT(importLandmarks()));
    fileMenu->addAction(tr("&Export Landmarks..."), this, SLOT(exportLandmarks()));
    fileMenu->addAction(tr("E&xit"), this, SLOT(close()), QKeySequence(QKeySequence::Quit));

    QMenu *manageMenu = menuBar()->addMenu(tr("&Manage"));
    manageMenu->addAction(tr("&Clear Landmarks"), this, SLOT(clearLandmarks()));

    landmarkCombo = new QComboBox();
    landmarkCombo->setSizePolicy(
        QSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Preferred));
    connect(landmarkCombo, SIGNAL(currentIndexChanged(int)),
            this, SLOT(showLandmark(int)));
    updateLandmarks();

    QWidget *widget = new QWidget();
    QFormLayout *layout = new QFormLayout(widget);
    layout->addRow(view);
    layout->addRow(tr("&Landmark:"), landmarkCombo);

    if (!landmarkManager->landmarks().isEmpty())
        showLandmark(0);

    setCentralWidget(widget);
    setWindowTitle(tr("Landmarks"));
}
    
void MapWindow::showLandmark(int index)
{
    if (index == -1)
        return;

    QLandmark landmark = landmarkManager->landmarks()[index];
    geoMap->setCenter(landmark.coordinate());
    geoMap->setZoomLevel(14);
}

void MapWindow::updateLandmarks()
{
    landmarkCombo->clear();
    foreach (QGeoMapObject *obj, geoMap->mapObjects())
        geoMap->removeMapObject(obj);

    foreach (QLandmark landmark, landmarkManager->landmarks()) {

        landmarkCombo->addItem(landmark.name());
        QGeoMapPixmapObject *pixmapObj = new QGeoMapPixmapObject();
        pixmapObj->setCoordinate(landmark.coordinate());
        pixmapObj->setOffset(QPoint(-2, -30));
        pixmapObj->setPixmap(QPixmap(":Resources/landmark.png"));
        geoMap->addMapObject(pixmapObj);
    }
}

void MapWindow::importLandmarks()
{
    path = QFileDialog::getOpenFileName(this, tr("Import Landmarks"),
        path, tr("Landmark files (*.gpx *.lmx)"));

    if (path.isEmpty())
        return;

    QFile file(path);
    if (!landmarkManager->importLandmarks(&file))
        QMessageBox::warning(this, tr("Landmarks"),
            tr("Failed to import landmarks: \"%1\"").arg(landmarkManager->errorString()));
    else
        updateLandmarks();
}

void MapWindow::exportLandmarks()
{
    path = QFileDialog::getSaveFileName(this, tr("Export Landmarks"),
        path, tr("Landmark files (*.gpx *.lmx)"));

    if (path.isEmpty())
        return;

    QString suffix = QFileInfo(path).suffix();
    suffix = suffix[0].toUpper() + suffix.mid(1);

    QFile file(path);
    if (!landmarkManager->exportLandmarks(&file, suffix))
        QMessageBox::warning(this, tr("Landmarks"),
            tr("Failed to export landmarks: \"%1\"").arg(landmarkManager->errorString()));
}

void MapWindow::clearLandmarks()
{
    int answer = QMessageBox::question(this, tr("Landmarks"),
        tr("Remove all stored landmarks, including those stored persistently "
           "in the database?"), QMessageBox::Yes, QMessageBox::No);

    if (answer == QMessageBox::Yes) {
        landmarkManager->removeLandmarks(landmarkManager->landmarks());
        updateLandmarks();
    }
}
